@@ -51,7 +51,7 @@ sub get_request_uri
 
 sub get_libs
 {
-    do("$ENV{'THEME_ROOT'}/authentic-funcs.pl");
+    do($ENV{'THEME_ROOT'} . "/authentic-funcs.pl");
     do(get_env('document_root') . '/' . get_module() . '/' . get_module() . '-lib.pl');
 
     ReadParse();
@@ -140,7 +140,7 @@ sub folders_select
             $offset++;
         } else {
             utf8::decode($f->{'name'}); utf8::encode($f->{'name'});
-            push(@opts, [$id, $f->{'name'}]);
+            push(@opts, [$id, html_escape($f->{'name'})]);
         }
     }
     return ui_select(undef, undef, \@opts);
@@ -312,9 +312,9 @@ sub message_details
              [$text{'extensions_mail_header_signed_by'} . ":",
               ui_italic(($dkim_signature ? $dkim_signature : $fail), 'light')
              ]
-        ) if ($dkim_signature);
+        );
         push(@tcontent, [$text{'extensions_mail_header_dkim'} . ":", ui_italic(($dkim ? $success : $fail), 'light')])
-          if ($dkim_signature && $dkim || (!$dkim_signature && !$dkim));
+          if ($dkim_signature && $dkim);
         push(@tcontent, [$text{'extensions_mail_header_spf'} . ":", ui_italic(($spf ? $success : $fail), 'light')]);
         push(@tcontent,
              [$text{'extensions_mail_header_encrypted'} . ":",
@@ -795,7 +795,7 @@ sub ui_table_content
         $content .= "<tr>";
         foreach my $cell (@$row) {
             $content .= "<td>";
-            $content .= $cell;
+            $content .= html_escape($cell);
             $content .= "</td>";
         }
         $content .= "</tr>";
