@@ -262,6 +262,10 @@ sub nav_detector
     # Temporary patch to address older, existing user configuration
     $nav_def_tab = $prd_mb if ($nav_def_tab eq 'mail');
 
+    # If Dashboard mode is set
+    $nav_def_tab = "$1" eq $mod_vm ? $prd_vm : $prd_cm
+        if ($page =~ /$page_sysinfo\?[\w]{3}\=($mod_vm|$mod_cm)/);
+
     # Return detected tab and page
     $tab  = $nav_def_tab;
     $page = $page;
@@ -331,15 +335,15 @@ sub nav_virtualmin_menu
     my @menu   = list_combined_webmin_menu({ 'dom' => "$def" }, \%in, $mod);
     my $menu   = nav_list_combined_menu([$mod], \@menu, undef, undef, $page);
     my $rv     = $menu->{'before'};
-    $rv .= nav_link_sysinfo();
+    $rv .= nav_link_sysinfo(undef, $mod);
     $rv .= nav_link_sysstat();
     $rv .= $menu->{'after'}
       if ($menu->{'after'});
     $rv .= nav_theme_links();
     $rv .= nav_links($menu->{'mode'});
     $rv .= nav_menu_html_snippet();
-    $rv .= nav_detect_page($page);
-    $rv .= nav_detect_script();
+    $rv .= nav_detect_page($page, 1);
+    $rv .= nav_detect_script(1);
     return $rv;
 }
 
@@ -351,7 +355,7 @@ sub nav_cloudmin_menu
     my @menu   = list_combined_webmin_menu({ 'server' => "$def" }, \%in, $mod);
     my $menu   = nav_list_combined_menu([$mod], \@menu, undef, undef, $page);
     my $rv     = $menu->{'before'};
-    $rv .= nav_link_sysinfo();
+    $rv .= nav_link_sysinfo(undef, $mod);
     $rv .= nav_theme_links();
     $rv .= nav_links($menu->{'mode'});
     $rv .= nav_menu_html_snippet();
@@ -427,14 +431,27 @@ sub nav_get_server_id
     return $default;
 }
 
+sub nav_set_last_id
+{
+    my ($link) = @_;
+    if ($link && $link =~ /(id|dom)=(\d+)/) {
+        return "vars.navigation.select.last='$2';";
+    }
+    return undef;
+}
+
 sub nav_detect_page
 {
-    my ($page) = @_;
+    my ($page, $set_last) = @_;
     my $rv;
     if ($page) {
         $page = quote_escape($page);
         $page =~ s/&amp;/&/g;
-        $rv .= "<li data-goto=\"$page\" class=\"hidden\"></li>\n";
+        my $last_id = $set_last ? nav_set_last_id($page) : undef;
+        if ($last_id) {
+            $last_id = "<script>$last_id</script>";
+        }
+        $rv .= "<li data-goto=\"$page\" class=\"hidden\">$last_id</li>\n";
     }
     return $rv;
 }
@@ -442,12 +459,14 @@ sub nav_detect_page
 sub nav_detect_script
 {
     # Get goto hidden li element
+    my ($set_last) = @_;
     my $rv;
     if ($server_x_goto) {
         my $link = quote_escape($server_x_goto);
         $link =~ s/&amp;/&/g;
+        my $last_id = $set_last ? nav_set_last_id($link) : undef;
         $rv =
-"<li data-script-goto><script>plugins.navigation.detect(\"$link\");plugins.navigation.detect(\"$link\", 1);\$(\"li[data-script-goto]\").remove();</script></li>\n";
+"<li data-script-goto><script>plugins.navigation.detect(\"$link\");plugins.navigation.detect(\"$link\", 1);\$(\"li[data-script-goto]\").remove();${last_id}</script></li>\n";
     }
     return $rv;
 }
@@ -479,7 +498,7 @@ sub nav_cat_link
     $link = "/$link" if (!$external_link && $link !~ /^\//);
     $rv   = '<li data-linked' . ($hidden && ' class="hidden"') . '>' . "\n";
     $rv .= '<a' . ($hidden && ' data-parent-hidden') . ' href="' .
-      ($external_link ? $link : "$theme_webprefix$link") . '"> ' . $label . '</a>' . "\n";
+      ($external_link ? $link : "$theme_webprefix$link") . '"> ' . &filter_javascript($label) . '</a>' . "\n";
     $rv .= "</li>\n";
     return $rv;
 }
@@ -532,12 +551,17 @@ sub nav_cat
                       'cat_logs'          => 'fa-file-text',
                       'global_services'   => 'fa-puzzle-piece',
                       'cat_services'      => 'fa-puzzle-piece',
+                      'cat_dns'           => 'fa2 fa2-dns scaled1_5',
+                      'cat_dnsreg'        => 'fa2 fa2-domain scaled1_5',
+                      'cat_mail'          => 'fa2 fa2-email-open scaled1_5',
+                      'cat_web'           => 'fa2 fa2-web scaled1_5',
                       'create_new'        => 'fa-plus',
                       'create_add'        => 'fa-plus',
                       'create_create'     => 'fa-server-add',
                       'global_gce'        => 'fa-google',
                       'global_ec2'        => 'fa2 fa2-amazon scaled1_5',
                       'global_hosts'      => 'fa-globe',
+                      'global_docker'     => 'fa2 fa2-docker margined-left--1 margined-right-1',
                       'global_virtualmin' => 'fa-virtualmin scaled1_5',
                       'global_owners'     => 'fa-users',
                       'global_monitor'    => 'fa2 fa2-system',
@@ -557,7 +581,7 @@ sub nav_cat
         # Show link to close or open catgory
         $rv = "<li class=\"has-sub\">\n";
         $rv .= "<a data-has-sub-link href=\"#$c\">";
-        $rv .= "<i class=\"fa $icon fa-fw\"></i> <span>$label</span></a>\n";
+        $rv .= "<i class=\"fa $icon fa-fw\"></i> <span>@{[&filter_javascript($label)]}</span></a>\n";
         $rv .= '</li>' . "\n";
         return $rv;
     }
@@ -574,6 +598,7 @@ sub nav_search
 "<input type=\"text\" class=\"form-control sidebar-search\" name=\"search\" placeholder=\"$theme_text{'left_search'}\">\n";
         $rv .= "<i class=\"fa fa-search\"></i>\n";
         $rv .= "</div>\n";
+        $rv .= "<div class='autocomplete-dropdown-container'></div>\n";
         $rv .= "</form>\n</li>\n";
     }
     return $rv;
@@ -581,9 +606,10 @@ sub nav_search
 
 sub nav_link_sysinfo
 {
-    my ($user) = @_;
+    my ($user, $mod) = @_;
+    $mod = "?mod=$mod" if ($mod);
     return
-      nav_menu_link('/sysinfo.cgi',
+      nav_menu_link("/sysinfo.cgi$mod",
                     $user ? $theme_text{'body_header1'} : $theme_text{'theme_xhred_titles_dashboard'},
                     ($user ? 'fa-user-circle' : 'fa-dashboard'),
                     $theme_config{'settings_sysinfo_link_mini'} eq 'true', 1)
@@ -643,14 +669,20 @@ sub nav_theme_links
                         $target = " target=\"$target\"";
                     }
                     if ($e->{"port"}) {
-                        my $host = get_env('http_host');
-                        $host =~ s/:(\d+)$/:$e->{'port'}/;
-                        $e->{"link"} = "//$host$e->{'link'}";
+                        my $host = $e->{"host"};
+                        if (!$host) {
+                            $host = get_env('http_host');
+                            $host =~ s/:(\d+)$/:$e->{'port'}/;
+                        } else {
+                            $host .= ":$e->{'port'}";
+                        }
+                        my $proto = $e->{'protocol'} ? "$e->{'protocol'}://" : "//";
+                        $e->{"link"} = "$proto$host$e->{'link'}";
                     }
                     my $type       = string_contains($e->{'link'}, '&#47;&#47') ? '' : 'data-linked';
                     my $type_class = $type ? "navigation_module_trigger"             : "navigation_external_link";
                     $rv .= '<li ' . $type . ' data-after><a ' . $target . ' href="' .
-                      $e->{"link"} . '" class="' . $type_class . '"><i class="fa fa-fw fa-' . $e->{"icon"} . '"></i> <span>';
+                      quote_escape($e->{"link"}, '"') . '" class="' . $type_class . '"><i class="fa fa-fw fa-' . $e->{"icon"} . '"></i> <span>';
                     utf8::encode($e->{'title'});
                     $rv .= $e->{'title'};
                     $rv .= '</span></a></li>';
@@ -753,7 +785,7 @@ sub nav_list_combined_menu
 
                 } elsif ($link =~ /\/virtual-server\/list_users\.cgi/) {
                     $icon = '<i class="fa fa-fw fa2 fa2-users-cog"></i>';
-                } elsif ($link =~ /\/virtual-server\/list_aliases\.cgi/) {
+                } elsif ($link =~ /\/virtual-server\/list_aliases\.cgi/ && $id ne 'cat_mail') {
                     $icon = '<i class="fa fa-fw fa2 fa2-maillist"></i>';
                 } elsif ($link =~ /\/virtual-server\/list_databases\.cgi/) {
                     $icon = '<i class="fa fa-fw fa-database"></i>';
@@ -764,6 +796,9 @@ sub nav_list_combined_menu
                 } elsif ($link =~ /\/filemin\/index\.cgi/) {
                     $icon = '<i class="fa fa-fw fa-file-manager scaled2"></i>';
 
+                } elsif ($link =~ /\/xterm\/index\.cgi\?user/) {
+                    $icon = '<i class="fa fa2 fa-fw fa2-terminal fa-1_10x margined-right--2"></i>';
+
                 } elsif ($link =~ /\/virtual-server\/edit_html\.cgi/) {
                     $icon = '<i class="fa fa-fw fa-globe"></i>';
                 } elsif ($link =~ /\/server-manager\/edit_pass\.cgi/ ||
@@ -841,8 +876,13 @@ sub nav_list_combined_menu
                 }
 
             } elsif ($item->{'type'} eq 'html') {
-                $rv .= '<li class="menu-container menu-status"><span class="badge"><i class="fa2 fa-fw fa2-pulsate"></i>' .
-                  $item->{'html'} . '</span></li>';
+                if (trim(html_strip($item->{'html'}))) {
+                    $rv .= '<li class="menu-container menu-status"><span class="badge"><i class="fa2 fa-fw fa2-pulsate"></i>' .
+                      $item->{'html'} . '</span></li>';
+                }
+            } elsif ($item->{'type'} eq 'text' && $item->{'json'}) {
+                    $login_mode =
+"<span><strong>$theme_text{'theme_global_access_level'}</strong>:&nbsp;&nbsp;<em>@{[html_escape($item->{'json'}->{'level'})]}</em></span>";
             } elsif ($item->{'type'} eq 'text' && $item->{'desc'}) {
                 if ($login_mode++ == 1) {
                     $login_mode =
@@ -920,7 +960,7 @@ sub nav_list_combined_menu
                                  "title=\"" . virtual_server::show_domain_name($_) . "\""]
                               }
                               grep {virtual_server::can_edit_domain($_)}
-                              sort {$a->{'dom'} cmp $b->{'dom'}} virtual_server::list_domains()];
+                              sort {$a->{'dom'} cmp $b->{'dom'}} virtual_server::list_visible_domains()];
                     }
                     my $data_title =
                       $item->{'module'} eq 'virtual-server' ? $theme_text{'right_fdoms'} : $theme_text{'right_fvm2'};
@@ -958,7 +998,7 @@ sub nav_links
         $rv .= '<li data-linked' .
           get_button_tooltip('theme_xhred_titles_dashboard', 'settings_hotkey_sysinfo', 'auto top') . ' class="user-link">';
         $rv .=
-          '<a class="menu-exclude-link sidebar_sysinfo_link" href="' .
+          '<a tabindex="-1" class="menu-exclude-link sidebar_sysinfo_link" href="' .
           $theme_webprefix . '/sysinfo.cgi"><i class="fa fa-fw fa-' .
           ($get_user_level eq '3' ? 'user-circle' : 'dashboard') . '"></i></a>';
         $rv .= '</li>';
@@ -976,12 +1016,16 @@ sub nav_links
     $rv .= '<span><i class="fa fa-fw ' . (theme_night_mode() ? 'fa-sun' : 'fa-moon') . '"></i></span>';
     $rv .= '</li>';
 
-    if ($theme_config{'settings_show_terminal_link2'} ne 'false' &&
-        foreign_available("shell"))
+    if ((foreign_available("xterm") && $theme_config{'settings_show_terminal_link'} ne 'false') ||
+        (!foreign_available("xterm") && foreign_available("shell") && $theme_config{'settings_show_terminal_link2'} ne 'false'))
     {
-        $rv .= '<li data-linked' . get_button_tooltip('theme_tooltip_terminal_link2', 'settings_hotkey_shell2', 'auto top') .
+        my $t = foreign_available("xterm") ? undef : '2';
+        my $t_icons = !$t ? 'fa2 fa2-terminal' : 'fa fa-terminal';
+        $rv .=
+          '<li data-linked' .
+          get_button_tooltip('theme_tooltip_terminal_link' . $t . '', 'settings_hotkey_shell' . $t . '', 'auto top') .
           ' class="user-link ported-console cursor-pointer">';
-        $rv .= '<span><i class="fa fa-fw fa-terminal"></i></span>';
+        $rv .= "<span><i class=\"$t_icons fa-fw\"></i></span>";
         $rv .= '</li>';
     }
 
@@ -992,14 +1036,14 @@ sub nav_links
     $rv .= '<span><i class="fa fa-fw fa-star"></i></span>';
     $rv .= '</li>';
 
-    if ((&webmin_user_is_admin() && $theme_config{'settings_theme_options_button'} ne 'false') ||
+    if ((&webmin_user_is_admin()  && $theme_config{'settings_theme_options_button'} ne 'false') ||
         (!&webmin_user_is_admin() &&
             $theme_config{'settings_theme_config_admins_only_privileged'} ne 'true' &&
             $theme_config{'settings_theme_options_button'} ne 'false'))
     {
         my $tooltip = get_button_tooltip('settings_title', 'settings_hotkey_open_module_config_privileged', 'auto top');
         $rv .= "<li $tooltip data-linked class=\"user-link theme-options cursor-pointer\">";
-        $rv .= '<a class="menu-exclude-link" href="' . $theme_webprefix .
+        $rv .= '<a tabindex="-1" class="menu-exclude-link" href="' . $theme_webprefix .
           '/tconfig.cgi" data-href="' . $theme_webprefix . '/tconfig.cgi"><i class="fa2 fa-fw fa2-palette"></i></a>';
         $rv .= '</li>';
     }
@@ -1009,7 +1053,7 @@ sub nav_links
     {
         $rv .=
           '<li data-linked' . get_button_tooltip('theme_xhred_title_language_locale', undef, 'auto top') .
-          ' class="user-link"><a class="menu-exclude-link" href="' .
+          ' class="user-link"><a tabindex="-1" class="menu-exclude-link" href="' .
           $theme_webprefix . '/change-user"><i class="fa fa-fw fa-globe"></i></a></li>';
     }
 
@@ -1037,11 +1081,12 @@ sub nav_links
     my $menu_lnk_colla     = $theme_config{'settings_collapse_navigation_link'} eq 'true';
     my $menu_lnk_dashb     = $theme_config{'settings_sysinfo_link_mini'} eq 'true';
     my $menu_lnk_night     = $theme_config{'settings_show_night_mode_link'} eq 'true';
-    my $menu_lnk_term      = $theme_config{'settings_show_terminal_link2'} eq 'true';
     my $menu_lnk_favor     = $theme_config{'settings_favorites'} eq 'true';
     my $menu_lnk_themeconf = $theme_config{'settings_theme_options_button'} eq 'true';
     my $menu_lnk_lang      = $theme_config{'settings_leftmenu_button_language'} eq 'true';
     my $menu_lnk_refresh   = $theme_config{'settings_leftmenu_button_refresh'} eq 'true';
+    my $menu_lnk_term      = $theme_config{'settings_show_terminal_link'} eq 'true' ||
+      $theme_config{'settings_show_terminal_link2'} eq 'true';
 
     # Link buttons widths
     my $menu_width_needed = 0;
@@ -1076,13 +1121,13 @@ sub nav_links
     $rv .= "$menu_elem_br<li $user_title class=\"user-link user-link-acl$cursor_def\">";
     if ($foreign_acl) {
         $rv .=
-          '<a class="menu-exclude-link" data-href="' . $theme_webprefix . '/acl/edit_user.cgi" href="' .
+          '<a tabindex="-1" class="menu-exclude-link" data-href="' . $theme_webprefix . '/acl/edit_user.cgi" href="' .
           $theme_webprefix . '/acl/edit_user.cgi?user=' . (get_env('base_remote_user') eq "root" ? "root" : $remote_user) .
           '"><i class="fa2 fa-fw ' . get_user_icon() . '"></i>&nbsp;<span>' . $remote_user . '</span></a>';
     } else {
         $rv .=
-          '<a class="menu-exclude-link cursor-default no-hover"><i class="fa2 fa-fw ' .
-          get_user_icon() . '"></i>&nbsp;<span class="pointer-events-none">' . $remote_user . '</span></a>';
+          '<a tabindex="-1" class="menu-exclude-link cursor-default no-hover"><i class="fa2 fa-fw ' .
+          get_user_icon() . '"></i>&nbsp;<span>' . $remote_user . '</span></a>';
     }
     $rv .= '</li>';
 
@@ -1094,26 +1139,26 @@ sub nav_links
         get_env('http_user_agent') !~ /webmin/i)
     {
         my $tooltip = get_button_tooltip(($main::session_id ? 'theme_tooltip_logout' : 'theme_xhred_tooltip_switch_user'),
-                                         undef, 'auto top');
+                                         'settings_hotkey_logout_dbl', 'auto top');
         $rv .= "<li $tooltip class=\"user-link __logout-link\">";
         if ($main::session_id) {
             $rv .=
-              '<a data-nref class="menu-exclude-link" href="' .
+              '<a tabindex="-1" data-nref class="menu-exclude-link" href="' .
               $theme_webprefix . '/session_login.cgi?logout=1"><i class="fa fa-fw fa-sign-out text-danger"></i></a>';
         } else {
             $rv .=
-              '<a data-nref class="menu-exclude-link" href="' .
+              '<a tabindex="-1" data-nref class="menu-exclude-link" href="' .
               $theme_webprefix . '/switch_user.cgi"><i class="fa fa-fw fa-exchange text-danger"></i></a>';
         }
         $rv .= '</li>';
     } else {
         if ($theme_server_webprefix) {
-            my $master_link           = get_env('http_webmin_servers');
+            my $master_link           = "$gconfig{'webprefix'}/servers/";
             my $tooltip_go_to_master  = get_button_tooltip('tooltip_back_to_servers_index_master', undef, 'auto top');
             my $tooltip_other_servers = get_button_tooltip('tooltip_list_other_servers',           undef, 'auto top');
             $rv .= "<li class=\"user-link servers-index-link\">";
             $rv .=
-              '<a ' . $tooltip_go_to_master . ' data-nref class="menu-exclude-link" href="' .
+              '<a tabindex="-1" ' . $tooltip_go_to_master . ' data-nref class="menu-exclude-link" href="' .
               $master_link . '?' . $xnav . '"><i class="fa fa-fw fa2 fa2-server"></i></a>';
             $rv .= '
                 <div data-http-webmin-servers="' .
@@ -1132,7 +1177,7 @@ sub nav_links
       '<li data-linked' .
       get_button_tooltip('theme_xhred_filemanager_context_refresh', 'settings_hotkey_reload', 'auto top') .
       ' class="user-link' . ($theme_config{'settings_leftmenu_button_refresh'} ne 'true' && ' hidden') .
-'"><a class="menu-exclude-link" data-refresh="true" style="cursor: pointer"><i class="fa fa-fw fa-refresh"></i></a></li>';
+'"><a tabindex="-1" class="menu-exclude-link" data-refresh="true" style="cursor: pointer"><i class="fa fa-fw fa-refresh"></i></a></li>';
     $rv .= '</ul></li>';
     $rv .= "\n";
     return $rv;
@@ -1342,7 +1387,7 @@ sub print_switch
 
     }
     print '<a></a>
-            </div><div class="toggle-space"></div>';
+            </div><div class="toggle-space"></div><div class="select-dropdown-container"></div>';
 }
 
 # XXX - needs further refactor
@@ -1400,6 +1445,8 @@ sub nav_menu_html_snippet
     my $rv;
     my $html_snippet         = $theme_config{'settings_leftmenu_user_html'};
     my $html_snippet_limited = $theme_config{'settings_leftmenu_user_html_privileged'};
+    $html_snippet = "<i class='fa2 fa-2x-force fa2-bug opacity-0_2'></i>"
+        if (!$html_snippet && theme_debug_mode());
     $html_snippet =~ s/(<(\/|\s*)(html|head|meta|link|title|body).*?>)//g;
 
     if ($html_snippet_limited ne 'true' ||
