@@ -120,7 +120,9 @@ while($line = <$FILE>) {
 close($FILE);
 
 # parse into data structures
-my $i = 0; my $num = 0;
+my $i = 0;
+my $num = 0;
+my $currttl = undef;
 while($i < @tok) {
 	if ($tok[$i] =~ /^\$origin$/i) {
 		# $ORIGIN directive (may be relative or absolute)
@@ -179,6 +181,7 @@ while($i < @tok) {
 		       	       'defttl' => $tok[$i++],
 			       'type' => '' };
 		push(@rv, $defttl);
+		$currttl = $defttl->{'defttl'};
 		}
 	elsif ($tok[$i] =~ /^\$(\S+)/i) {
 		# some other special directive
@@ -279,6 +282,7 @@ while($i < @tok) {
 			}
 		$dir{'values'} = \@values;
 		$dir{'eline'} = $lnum[$i-1];
+		$dir{'realttl'} = $dir{'ttl'} || $currttl;
 
 		# Work out canonical form, and maybe use it
 		my $canon = $dir{'name'};
@@ -491,11 +495,10 @@ return $name.
 # Increase the serial number in some SOA record by 1
 sub bump_soa_record
 {
-my($r, $v, $vals);
-for(my $i=0; $i<@{$_[1]}; $i++) {
-	$r = $_[1]->[$i];
+my ($file, $recs) = @_;
+foreach my $r (@$recs) {
 	if ($r->{'type'} eq "SOA") {
-		$v = $r->{'values'};
+		my $v = $r->{'values'};
 		# already set serial if no acl allow it to update or update
 		# is disabled
 		my $serial = $v->[2];
@@ -503,8 +506,8 @@ for(my $i=0; $i<@{$_[1]}; $i++) {
 			# automatically handle serial numbers ?
 			$serial = &compute_serial($v->[2]);
 			}
-		$vals = "$v->[0] $v->[1] (\n\t\t\t$serial\n\t\t\t$v->[3]\n".
-			"\t\t\t$v->[4]\n\t\t\t$v->[5]\n\t\t\t$v->[6] )";
+		my $vals = "$v->[0] $v->[1] (\n\t\t\t$serial\n\t\t\t$v->[3]\n".
+			   "\t\t\t$v->[4]\n\t\t\t$v->[5]\n\t\t\t$v->[6] )";
 		&modify_record($r->{'file'}, $r, $r->{'realname'}, $r->{'ttl'},
 				$r->{'class'}, $r->{'type'}, $vals);
 		}
@@ -927,7 +930,8 @@ else {
 	# All one one line
 	my @rv;
 	foreach my $v (@{$r->{'values'}}) {
-		push(@rv, $v =~ /\s|;/ ? "\"$v\"" : $v);
+		push(@rv, $v =~ /\s|;/ || $r->{'type'} eq 'TXT' ?
+				"\"$v\"" : $v);
 		}
 	return join(" ", @rv);
 	}
