@@ -4,7 +4,6 @@ BEGIN { push(@INC, ".."); };
 use WebminCore;
 &init_config();
 
-use lib './lib';
 use POSIX;
 use Encode qw(decode encode);
 use File::Basename;
@@ -52,6 +51,7 @@ sub get_paths {
     elsif ($access{'work_as_root'}) {
         # Root user, so no switching
         @remote_user_info = getpwnam('root');
+	@WebminCore::remote_user_info = @remote_user_info;
     }
     elsif ($access{'work_as_user'}) {
         # A specific user
@@ -59,6 +59,7 @@ sub get_paths {
         @remote_user_info ||
             &error("Unix user $access{'work_as_user'} does not exist!");
         &switch_to_unix_user(\@remote_user_info);
+	@WebminCore::remote_user_info = @remote_user_info;
     }
     else {
         # Run as the Webmin user we are connected as
@@ -100,8 +101,6 @@ sub get_paths {
     $quote_escaped_path = quote_escape($path);
     $urlized_path = urlize($path);
     
-    $cwd = &simplify_path($base.$path);
-
     # Work out max upload size
     if (&get_product_name() eq 'usermin') {
 	$upload_max = $config{'max'};
@@ -110,6 +109,7 @@ sub get_paths {
     }
 
     # Check that current directory is one of those that is allowed
+    $cwd = &simplify_path($base.$path);
     my $error = 1;
     for $allowed_path (@allowed_paths) {
         if (&is_under_directory($allowed_path, $cwd) ||
@@ -152,14 +152,15 @@ sub print_template {
 }
 
 sub print_errors {
-    my @errors = @_;
+    my (@errors) = @_;
     &ui_print_header(undef, $module_info{'name'}, "");
-    print $text{'errors_occured'};
-    print "<ul>";
+    print "<tt>$text{'errors_occured'}</tt><br>";
+    print "<ul class=\"err-body\">";
     foreach $error(@errors) {
-        print("<li>$error</li>");
+        print("<li><tt>$error</tt></li>");
     }
-    print "<ul>";
+    print "</ul>";
+    print "<script>if(typeof print_errors_post==='function'){print_errors_post('$module_name')}</script>";
     &ui_print_footer("index.cgi?path=".&urlize($path), $text{'previous_page'});
 }
 
@@ -168,9 +169,23 @@ sub print_interface {
     local $upid = time().$$;
     $bookmarks = get_bookmarks();
     @allowed_for_edit = split(/\s+/, $access{'allowed_for_edit'});
+    # Some experimental MIME types are now recognized
+    push(@allowed_for_edit, map { (my $__ = $_) =~ s/-x-/-/;
+        $__ ne $_ ? $__ : () } @allowed_for_edit);
     %allowed_for_edit = map { $_ => 1} @allowed_for_edit;
     my %tinfo = &get_theme_info($current_theme);
 
+    # User and group lists for acls
+    if (&has_command('setfacl')) {
+        our $acl_user_select = &ui_user_textbox("user", $realuser);
+        our $acl_group_select = &ui_user_textbox("group", $realuser);
+        our $acl_manual = &ui_details(
+            { title => $text{'acls_manual'},
+              content => &ui_textbox("manual", undef, 40,
+                    undef, undef, "placeholder='-m u:root:rw-,g:stream:r-x -R'"),
+              html => 1 } );
+        }
+
     # Interface for Bootstrap powered themes
     if ($tinfo{'bootstrap'}) {
 
@@ -385,6 +400,7 @@ sub print_interface {
                    index($type, "-x-tar") != -1           ||
                    (index($type, "-x-bzip") != -1 && has_command('bzip2')) ||
                    (index($type, "-gzip") != -1   && has_command('gzip'))  ||
+                   (index($type, "zstd") != -1   && has_command('zstd'))  ||
                    (index($type, "-x-xz") != -1   && has_command('xz'))
                   ) &&
                   has_command('tar')))
