@@ -4,7 +4,6 @@
 #
 # Rene Mayrhofer, July 2000
 # Some code has been taken from redhat-linux-lib.pl
-
 $network_interfaces_config = '/etc/network/interfaces';
 $modules_config = '/etc/modprobe.d/arch/i386';
 if (!-d $modules_config) {
@@ -24,7 +23,7 @@ my @autos = &get_auto_defs();
 my @rv;
 my %v6map;
 foreach $iface (@ifaces) {
-	my ($name, $addrfam, $method, $options) = @$iface;
+	my ($name, $addrfam, $method, $options, $file, $lnum) = @$iface;
 	if ($addrfam eq 'inet') {
 		# IPv4 interface .. parse and add to list
 		my $cfg = { };
@@ -91,11 +90,16 @@ foreach $iface (@ifaces) {
 				$cfg->{$param} = $value;
 				}
 			}
+		if ($cfg->{'address'} =~ /^(\S+)\/(\d+)$/) {
+			$cfg->{'address'} = $1;
+			$cfg->{'netmask'} = &prefix_to_mask($2);
+			}
 		$cfg->{'dhcp'} = ($method eq 'dhcp');
 		$cfg->{'bootp'} = ($method eq 'bootp');
 		$cfg->{'edit'} = ($cfg->{'name'} !~ /^ppp|lo/);
 		$cfg->{'index'} = scalar(@rv);	
-		$cfg->{'file'} = $network_interfaces_config;
+		$cfg->{'file'} = $file;
+		$cfg->{'line'} = $lnum;
 		push(@rv, $cfg);
 		}
 	elsif ($addrfam eq "inet6") {
@@ -255,8 +259,8 @@ if(($cfg->{'vlan'} == 1) && ($cfg->{'mtu'})) {
 
 # Find the existing interface section
 my @ifaces = get_interface_defs();
-my $found = 0;
-my $found6 = 0;
+my $found;
+my $found6;
 foreach $iface (@ifaces) {
 	local $address;
 	foreach my $opt (@{$iface->[3]}) {
@@ -267,19 +271,19 @@ foreach $iface (@ifaces) {
 		}
 	if ($iface->[0] eq $cfg->{'fullname'} && $iface->[1] eq 'inet') {
 		# Found interface to change
-		$found = 1;
+		$found = $iface->[4];
 		foreach my $o (@{$iface->[3]}) {
 			if ($o->[0] eq 'gateway' ||
 			    $o->[0] eq 'pre-up' && $o->[1] =~ /brctl/ ||
 			    $o->[0] =~ /^(pre-)?up$/ && $o->[1] =~ /ip\s+route/ ||
-			    $o->[0] eq 'post-up' && $o->[1] =~ /iptables-restore/) {
+			    $o->[0] eq 'post-up') {
 				push(@options, $o);
 				}
 			}
 		}
 	if ($iface->[0] eq $cfg->{'fullname'} && $iface->[1] eq 'inet6') {
 		# Found IPv6 block
-		$found6 = 1;
+		$found6 = $iface->[4];
 		}
 	}
 
@@ -307,11 +311,11 @@ else {
 	# Update existing section
 	if($in{'vlan'} == 1) {
 		&modify_interface_def($cfg->{'physical'}.'.'.$cfg->{'vlanid'},
-				      'inet', $method, \@options, 0);
+				      'inet', $method, \@options, 0, $found);
 		}
 	else {
 		&modify_interface_def($cfg->{'fullname'},
-				      'inet', $method, \@options, 0);
+				      'inet', $method, \@options, 0, $found);
 		}
 	if ($cfg->{'bond'} == 1 && $gconfig{'os_version'} < 5) {
 		&modify_module_def($cfg->{'fullname'}, 0, $cfg->{'mode'},
@@ -345,16 +349,16 @@ my $method = $cfg->{'auto6'} ? "manual" : "static";
 if (!$found6 && @options6) {
 	# Need to add IPv6 block
 	&new_interface_def($cfg->{'fullname'},
-			   'inet6', $method, \@options6);
+			   'inet6', $method, \@options6, $found);
 	}
 elsif ($found6 && @options6) {
 	# Need to update IPv6 block
 	&modify_interface_def($cfg->{'fullname'},
-			      'inet6', $method, \@options6, 0);
+			      'inet6', $method, \@options6, 0, $found6);
 	}
 elsif ($found6 && !@options6) {
 	# Need to delete IPv6 block
-	&delete_interface_def($cfg->{'fullname'}, 'inet6');
+	&delete_interface_def($cfg->{'fullname'}, 'inet6', $found6);
 	}
 
 # Set auto option to include this interface, or not
@@ -516,9 +520,9 @@ sub new_module_def
 sub delete_interface
 {
 my $cfg = $_[0];
-&delete_interface_def($cfg->{'fullname'}, 'inet');
+&delete_interface_def($cfg->{'fullname'}, 'inet', $cfg->{'file'});
 if (@{$cfg->{'address6'}}) {
-	&delete_interface_def($cfg->{'fullname'}, 'inet6');
+	&delete_interface_def($cfg->{'fullname'}, 'inet6', $cfg->{'file'});
 	}
 my @autos = get_auto_defs();
 if ($gconfig{'os_version'} >= 3 || scalar(@autos)) {
@@ -531,7 +535,8 @@ if ($gconfig{'os_version'} >= 3 || scalar(@autos)) {
 # Can some boot-time interface parameter be edited?
 sub can_edit
 {
-return $_[0];
+my ($what) = @_;
+return 1;
 }
 
 sub can_broadcast_def
@@ -571,7 +576,14 @@ foreach my $f ("/etc/hostname", "/etc/HOSTNAME", "/etc/mailname") {
 		&close_tempfile(HOST);
 		}
 	}
-undef(@main::get_system_hostname);      # clear cache
+
+# Use the hostnamectl command as well
+if (&has_command("hostnamectl")) {
+	&system_logged("hostnamectl set-hostname ".quotemeta($hostname).
+		       " >/dev/null 2>&1");
+	}
+
+&get_system_hostname(undef, undef, 2);      # clear cache
 }
 
 # get_domainname()
@@ -592,7 +604,9 @@ my ($domain) = @_;
 
 sub routing_config_files
 {
-return ( $network_interfaces_config, $sysctl_config );
+my @defs = &get_interface_defs();
+my @files = &unique(map { $_->[4] } @defs);
+return ( @files, $sysctl_config );
 }
 
 sub network_config_files
@@ -770,59 +784,92 @@ $sysctl{'net.ipv4.ip_forward'} = $in{'forward'};
 # gets a list of interface definitions (including their options) from the
 # central config file
 # the returned list is an array whose contents are tupels of
-# (name, addrfam, method, options) with
+# (name, addrfam, method, options, file, line) with
 #    name          the interface name (e.g. eth0)
 #    addrfam       the address family (e.g. inet, inet6)
 #    method        the address activation method (e.g. static, dhcp, loopback)
 #    options       is a list of (param, value) pairs
+#    file          the file this interface was read from
+#    line          the line number in the file
 sub get_interface_defs
 {
-local *CFGFILE;
+my ($file, $done) = @_;
+$file ||= $network_interfaces_config;
+$done ||= { };
+return ( ) if ($done->{$file}++);
+my $fh;
 my @ret;
-&open_readfile(CFGFILE, $network_interfaces_config) || return ();
+open($fh, $file) || return ();
 # read the file line by line
-$line = <CFGFILE>;
+my $line = <$fh>;
+my $lnum = 0;
+my $err;
 while (defined $line) {
+	last if ($err);
 	chomp($line);
 	# skip comments
 	if ($line =~ /^\s*#/ || $line =~ /^\s*$/) {
-		$line = <CFGFILE>;
+		$line = <$fh>;
+		$lnum++;
 		next;
 		}
-
 	if ($line =~ /^\s*auto/) {
 		# skip auto stanzas
-		$line = <CFGFILE>;
+		$line = <$fh>;
+		$lnum++;
 		while(defined($line) && $line !~ /^\s*(iface|mapping|auto|source|allow-hotplug)/) {
-			$line = <CFGFILE>;
+			$line = <$fh>;
+			$lnum++;
 			next;
 			}
 		}
 	elsif ($line =~ /^\s*mapping/) {
 		# skip mapping stanzas
-		$line = <CFGFILE>;
+		$line = <$fh>;
+		$lnum++;
 		while(defined($line) && $line !~ /^\s*(iface|mapping|auto|source|allow-hotplug)/) {
-			$line = <CFGFILE>;
+			$line = <$fh>;
+			$lnum++;
 			next;
 			}
 		}
-	elsif ($line =~ /^\s*source/) {
-		# Skip includes
-		$line = <CFGFILE>;
+	elsif ($line =~ /^\s*(source|source-directory)\s+(\S+)/) {
+		# Expand includes
+		$line = <$fh>;
+		$lnum++;
+		my ($dir, $src) = ($1, $2);
+		my @srcs;
+		if ($dir eq "source-directory") {
+			opendir(SRCDIR, $src);
+			@srcs = grep { /^[a-zA-Z0-9_-]+$/ } readdir(SRCDIR);
+			closedir(SRCDIR);
+			@srcs = map { "$src/$_" } @srcs
+				if (@srcs);
+			}
+		else {
+			@srcs = glob($src);
+			}
+		foreach $src (@srcs) {
+			my @inc = &get_interface_defs($src, $done);
+			push(@ret, @inc);
+			}
 		}
 	elsif ($line =~ /^\s*allow-hotplug/) {
 		# Skip hotplug lines
-		$line = <CFGFILE>;
+		$line = <$fh>;
+		$lnum++;
 		}
 	elsif (my ($name, $addrfam, $method) = ($line =~ /^\s*iface\s+(\S+)\s+(\S+)\s+(\S+)\s*$/) ) {
 		# only lines starting with "iface" are expected here
 		my @iface_options;
 		# now read everything until the next iface definition
-		$line = <CFGFILE>;
+		$line = <$fh>;
+		$lnum++;
 		while (defined $line && ! ($line =~ /^\s*(iface|mapping|auto|source|allow-hotplug)/)) {
 			# skip comments and empty lines
 			if ($line =~ /^\s*#/ || $line =~ /^\s*$/) {
-				$line = <CFGFILE>;
+				$line = <$fh>;
+				$lnum++;
 				next;
 				}
 			my ($param, $value);
@@ -835,15 +882,18 @@ while (defined $line) {
 			else {
 				error("Error in option line: '$line' invalid");
 				}
-			$line = <CFGFILE>;
+			$line = <$fh>;
+			$lnum++;
 			}
-		push(@ret, [$name, $addrfam, $method, \@iface_options]);
+		push(@ret, [$name, $addrfam, $method, \@iface_options,
+			    $file, $lnum]);
 		}
 	else {
+		$err++;
 		error("Error reading file $network_interfaces_config: unexpected line '$line'");
 		}
 	}
-close(CFGFILE);
+close($fh);
 return @ret;
 }
 
@@ -851,12 +901,12 @@ return @ret;
 # Returns a list of interfaces in auto lines
 sub get_auto_defs
 {
-local @rv;
+my @rv;
 &open_readfile(CFGFILE, $network_interfaces_config);
 while(<CFGFILE>) {
 	s/\r|\n//g;
 	s/^\s*#.*$//g;
-	if (/^\s*auto\s*(.*)/) {
+	if (/^\s*auto\s*(\S.*)/) {
 		push(@rv, split(/\s+/, $1));
 		}
 	}
@@ -894,20 +944,21 @@ if (!$found) {
 }
 
 # modifies the options of an already stored interface definition
-# the parameters should be (name, addrfam, method, options, mode)
+# the parameters should be (name, addrfam, method, options, mode, file)
 # with options being an array of (param, value) pairs
 # and mode being 0 for modify and 1 for delete
 sub modify_interface_def
 {
-my ($name, $addrfam, $method, $options, $mode) = @_;
+my ($name, $addrfam, $method, $options, $mode, $file) = @_;
+$file ||= $network_interfaces_config;
 # make a backup copy
-&copy_source_dest($network_interfaces_config, "$network_interfaces_config~");
+&copy_source_dest($file, $file."~");
 local *OLDCFGFILE, *NEWCFGFILE;
-&open_readfile(OLDCFGFILE, "$network_interfaces_config~") ||
-	error("Unable to open $network_interfaces_config");
-&lock_file($network_interfaces_config);
-&open_tempfile(NEWCFGFILE, "> $network_interfaces_config", 1) ||
-	error("Unable to open $network_interfaces_config");
+&open_readfile(OLDCFGFILE, $file."~") ||
+	error("Unable to open $file");
+&lock_file($file);
+&open_tempfile(NEWCFGFILE, ">$file", 1) ||
+	error("Unable to open $file");
 
 my $inside_modify_region = 0;
 my $iface_line = 0;
@@ -935,7 +986,7 @@ while (defined ($line=<OLDCFGFILE>)) {
 		}
 	# inside modify region or not ?
 	elsif ($inside_modify_region == 0) {
-               &print_tempfile(NEWCFGFILE, $line);
+		&print_tempfile(NEWCFGFILE, $line);
 		}
 	else {
 		# should the iface line be changed or the options ?
@@ -948,45 +999,45 @@ while (defined ($line=<OLDCFGFILE>)) {
                if ($mode == 0 && $new_options_wrote == 0) {
                        $new_options_wrote = 1;
                        foreach $option (@$options) {
-                               my ($param, $value) = @$option;
-                               &print_tempfile(NEWCFGFILE, "\t$param $value\n");
+				my ($param, $value) = @$option;
+				&print_tempfile(NEWCFGFILE,"\t$param $value\n");
 				}
 			}
 		}
 	$iface_line = 0;
 	}
-
 close(OLDCFGFILE);
 &close_tempfile(NEWCFGFILE);
-&unlock_file($network_interfaces_config);
+&unlock_file($file);
+&unlink_file($file."~");
 }
 
 # creates a new interface definition in the config file
-# the parameters should be (name, addrfam, method, options)
+# the parameters should be (name, addrfam, method, options, file)
 # with options being an array of (param, value) pairs
 # the selection key is (name, addrfam)
 sub new_interface_def
 {
-# make a backup copy
-&copy_source_dest($network_interfaces_config, "$network_interfaces_config~");
+local ($name, $addrfam, $method, $options, $file) = @_;
+$file ||= $network_interfaces_config;
 local *CFGFILE;
-&open_lock_tempfile(CFGFILE, ">> $network_interfaces_config") ||
-	error("Unable to open $network_interfaces_config");
-local ($name, $addrfam, $method, $options) = @_;
+&open_lock_tempfile(CFGFILE, ">>$file") ||
+	error("Unable to open $file");
 &print_tempfile(CFGFILE, "\niface $name $addrfam $method\n");
 foreach $option (@$options) {
 	my ($param, $value) = @$option;
 	&print_tempfile(CFGFILE, "\t$param $value\n");
 	}
 &close_tempfile(CFGFILE);
+&unlock_file($file);
 }
 
 # delete an already defined interface
-# the parameters should be (name, addrfam)
+# the parameters should be (name, addrfam, file)
 sub delete_interface_def
 {
-local ($name, $addrfam, $method) = @_;
-&modify_interface_def($name, $addrfam, '', [], 1);
+local ($name, $addrfam, $file) = @_;
+&modify_interface_def($name, $addrfam, '', [], 1, $file);
 &modify_module_def($name, 1);
 }
 
@@ -1028,7 +1079,8 @@ foreach my $iface (grep { $_->[1] eq 'inet' } @ifaces) {
 	if ($iface->[0] eq $_[1]) {
 		push(@{$iface->[3]}, [ 'gateway', $_[0] ]);
 		}
-	&modify_interface_def(@$iface);
+	&modify_interface_def($iface->[0], $iface->[1], $iface->[2],
+			      $iface->[3], 0, $iface->[4]);
 	}
 }
 
@@ -1063,7 +1115,8 @@ foreach my $iface (grep { $_->[1] eq 'inet6' } @ifaces) {
 	if ($iface->[0] eq $_[1] && $_[0]) {
 		push(@{$iface->[3]}, [ 'gateway', $_[0] ]);
 		}
-	&modify_interface_def(@$iface);
+	&modify_interface_def($iface->[0], $iface->[1], $iface->[2],
+			      $iface->[3], 0, $iface->[4]);
 	}
 }
 
