@@ -182,20 +182,45 @@ elsif ($config{'mode'} == 3 && !$forglobal) {
 return \@rv;
 }
 
-# find(name, &config)
+# alt_name(name, &config)
+# Returns an alternate directive name if the current one is obsoleted
+sub alt_name
+{
+my ($name, $conf) = @_;
+my $newname = $name;
+if ($newname =~ s/welcomelist/welcomelist/ ||
+    $newname =~ s/blocklist/blocklist/) {
+	# Is the new directive being used?
+	my @olduse = &find($name, $conf, 1);
+	my @newuse = &find($newname, $conf, 1);
+	if (@newuse) {
+		# New name exists in config, so use it
+		$name = $newname;
+		}
+	elsif (!@olduse && &version_atleast("4.0")) {
+		# Neither exists, but use the new name is later versions
+		$name = $newname;
+		}
+	}
+return $name;
+}
+
+# find(name, &config, [no-alt])
 sub find
 {
-local @rv;
-foreach $c (@{$_[1]}) {
-	push(@rv, $c) if (lc($c->{'name'}) eq lc($_[0]));
+my ($name, $conf, $noalt) = @_;
+my @rv;
+$name = &alt_name($name, $conf) if (!$noalt);
+foreach my $c (@$conf) {
+	push(@rv, $c) if (lc($c->{'name'}) eq lc($name));
 	}
 return wantarray ? @rv : $rv[0];
 }
 
-# find_value(name, &config)
+# find_value(name, &config, [no-alt])
 sub find_value
 {
-local @rv = map { $_->{'value'} } &find(@_);
+my @rv = map { $_->{'value'} } &find(@_);
 return wantarray ? @rv : $rv[0];
 }
 
@@ -203,6 +228,7 @@ return wantarray ? @rv : $rv[0];
 # Update the config file with some directives
 sub save_directives
 {
+my ($conf, $nameold, $newv, $vonly) = @_;
 if ($module_info{'usermin'} && $local_cf =~ /^(.*)\/([^\/]+)$/) {
 	# Under Usermin, make sure .spamassassin exists
 	local $spamdir = $1;
@@ -210,11 +236,13 @@ if ($module_info{'usermin'} && $local_cf =~ /^(.*)\/([^\/]+)$/) {
 		&make_dir($spamdir, 0755);
 		}
 	}
-local @old = ref($_[1]) ? @{$_[1]} : &find($_[1], $_[0]);
-local @new = $_[3] ? &make_directives($_[1], $_[2]) : @{$_[2]};
-local $i;
-for($i=0; $i<@old || $i<@new; $i++) {
-	local $line;
+if (!ref($nameold)) {
+	$nameold = &alt_name($nameold, $conf);
+	}
+my @old = ref($nameold) ? @$nameold : &find($nameold, $conf);
+my @new = $vonly ? &make_directives($nameold, $newv) : @$newv;
+for(my $i=0; $i<@old || $i<@new; $i++) {
+	my $line;
 	if ($new[$i]) {
 		$line = $new[$i]->{'name'};
 		$line .= " ".$new[$i]->{'value'} if ($new[$i]->{'value'} ne '');
@@ -262,7 +290,7 @@ for($i=0; $i<@old || $i<@new; $i++) {
 				    $rv ? $rv->error : "Unknown modify error"));
 				}
 			}
-		$_[0]->[$old[$i]->{'index'}] = $new[$i];
+		$conf->[$old[$i]->{'index'}] = $new[$i];
 		}
 	elsif ($old[$i]) {
 		# Deleting a directive
@@ -270,7 +298,7 @@ for($i=0; $i<@old || $i<@new; $i++) {
 			# From a file
 			local $lref = &read_file_lines($old[$i]->{'file'});
 			splice(@$lref, $old[$i]->{'line'}, 1);
-			foreach $c (@{$_[0]}) {
+			foreach $c (@$conf) {
 				if ($c->{'line'} > $old[$i]->{'line'} &&
 				    $c->{'file'} eq $old[$i]->{'file'}) {
 					$c->{'line'}--;
@@ -308,8 +336,8 @@ for($i=0; $i<@old || $i<@new; $i++) {
 			}
 
 		# Fix up indexes
-		splice(@{$_[0]}, $old[$i]->{'index'}, 1);
-		foreach $c (@{$_[0]}) {
+		splice(@$conf, $old[$i]->{'index'}, 1);
+		foreach $c (@$conf) {
 			if ($c->{'index'} > $old[$i]->{'index'}) {
 				$c->{'index'}--;
 				}
@@ -353,17 +381,19 @@ for($i=0; $i<@old || $i<@new; $i++) {
 				}
 			}
 		$new[$i]->{'mode'} = $addmode;
-		$new[$i]->{'index'} = @{$_[0]};
-		push(@{$_[0]}, $new[$i]);
+		$new[$i]->{'index'} = @$conf;
+		push(@$conf, $new[$i]);
 		}
 	}
 }
 
 # make_directives(name, &values)
+# Converts a list of values into a list of directive objects
 sub make_directives
 {
-return map { { 'name' => $_[0],
-	       'value' => $_ } } @{$_[1]};
+my ($name, $vals) = @_;
+return map { { 'name' => $name,
+	       'value' => $_ } } @$vals;
 }
 
 ### UI functions ###
@@ -577,7 +607,7 @@ sub version_atleast
 if (!$version_cache) {
 	$version_cache = &get_spamassassin_version();
 	}
-return $version_cache >= $_[0];
+return &compare_version_numbers($version_cache, '>=', $_[0]);
 }
 
 # spam_file_folder()
@@ -633,7 +663,7 @@ print "<table width=100%> <tr>\n";
 local $onclick = "onClick='return check_clicks(form)'"
 	if (defined(&check_clicks_function));
 print "<td align=left><input type=submit name=inbox value=\"$text{'mail_inbox'}\" $onclick></td>\n";
-print "<td align=left><input type=submit name=whitelist value=\"$text{'mail_whitelist2'}\" $onclick></td>\n";
+print "<td align=left><input type=submit name=welcomelist value=\"$text{'mail_welcomelist2'}\" $onclick></td>\n";
 if (&has_command($config{'sa_learn'})) {
 	print "<td align=center><input type=submit name=ham value=\"$text{'mail_ham'}\" $onclick></td>\n";
 	}
@@ -966,19 +996,19 @@ $get_ldap_user_cache{$user} = $uinfo;
 return $uinfo;
 }
 
-# get_auto_whitelist_file([user])
-# Returns the base path to the auto whitelist DBM, if any. 
-sub get_auto_whitelist_file
+# get_auto_welcomelist_file([user])
+# Returns the base path to the auto welcomelist DBM, if any. 
+sub get_auto_welcomelist_file
 {
 local ($user) = @_;
 local @uinfo = $module_info{'usermin'} ? @remote_user_info :
 	       $user ? getpwnam($user) : ( );
 local $conf = &get_config();
-local $awp = &find("auto_whitelist_path", $conf);
+local $awp = &find("auto_welcomelist_path", $conf);
 if (!$awp) {
-	$awp = &find_default("auto_whitelist_path");
+	$awp = &find_default("auto_welcomelist_path");
 	}
-$awp ||= "~/.spamassassin/auto-whitelist";
+$awp ||= "~/.spamassassin/auto-welcomelist";
 if ($awp !~ /^\//) {
 	# Make absolute
 	return undef if (scalar(@uinfo) == 0);
@@ -999,13 +1029,13 @@ if (!&is_under_directory($uinfo[7], $awp)) {
 return $awp;
 }
 
-# open_auto_whitelist_dbm([user])
-# Ties the %awl hash to the autowhitelist DBM file. Returns 1 if successful, or
+# open_auto_welcomelist_dbm([user])
+# Ties the %awl hash to the autowelcomelist DBM file. Returns 1 if successful, or
 # 0 if it could not be opened, or -1 if empty.
-sub open_auto_whitelist_dbm
+sub open_auto_welcomelist_dbm
 {
 local ($user) = @_;
-local $awp = &get_auto_whitelist_file($user);
+local $awp = &get_auto_welcomelist_file($user);
 return 0 if (!$awp);
 local $anyok;
 foreach my $cls ('DB_File', 'GDBM_File', 'SDBM_File') {
@@ -1021,20 +1051,20 @@ foreach my $cls ('DB_File', 'GDBM_File', 'SDBM_File') {
 return $anyok ? -1 : 0;
 }
 
-# close_auto_whitelist_dbm()
+# close_auto_welcomelist_dbm()
 # Disconnects the global %awl hash from the DBM file, flushing changes to disk
-sub close_auto_whitelist_dbm
+sub close_auto_welcomelist_dbm
 {
 untie(%awl);
 }
 
-# supports_auto_whitelist()
-# Returns 1 if SpamAssassin is doing auto-whitelisting for the current user,
+# supports_auto_welcomelist()
+# Returns 1 if SpamAssassin is doing auto-welcomelisting for the current user,
 # 2 if for multiple users.
-sub supports_auto_whitelist
+sub supports_auto_welcomelist
 {
 if ($module_info{'usermin'}) {
-	return &get_auto_whitelist_file() ? 1 : 0;
+	return &get_auto_welcomelist_file() ? 1 : 0;
 	}
 else {
 	return 2;
